@@ -1,7 +1,8 @@
 #!/usr/bin/perl -w
 
 ###############################################################################
-# cvsplot: Copyright (c) 2001, 2002, 2003 David Sitsky.  All rights reserved.
+# cvsplot: Copyright (c) 2001, 2002, 2003, 2004 David Sitsky.
+# All rights reserved.
 #
 # cvsplot is a perl script which is used to extract information from CVS and
 # plots the total number of lines and number of files in a selected file set
@@ -15,20 +16,25 @@
 
 use Config;
 use Cwd;
-use Date::Manip;
-use String::ShellQuote qw(shell_quote);
 use Symbol;
 use IPC::Open3;
 
 # Whether debugging is enabled or not.
 $debug = 0;
 
+# Additional global arguments to use with CVS commands.
+$cvs_global_args = "";
+
 # The start date in which to gather statistics.
 $start_date = "";
 
 # The final date in which to gather statistics.
 $end_date = "";
 
+# Indicate whether to count the number of lines _changed_, or the
+# the number of lines added.
+$count_lines_changed = 0;
+
 # The directory is which to gather the cvs statistics (where the cvs log
 # command is run from), or the directory of the CVS repository, if the
 # -rlog option is used.
@@ -52,6 +58,18 @@ $linedata = "";
 # Where the number of files statistics will be stored.
 $filedata = "";
 
+# Where the per-user line statistics will be stored.
+$userdata = "";
+
+# A set of users whose data needs to be examined individually.
+%filterusers = ();
+
+# A map of a set of group to user memberships.
+%usergroups = ();
+
+# The name of the default user group (if any).
+$default_user_group = "";
+
 # A hash (by date) of a hash (by filename) of lines added.
 %line_stats = ();
 
@@ -61,12 +79,18 @@ $filedata = "";
 # A hash (by date) of a hash (by filename) of the revision.
 %revision_stats = ();
 
+# A hash (by date) of a hash (by filename) of the author.
+%author_stats = ();
+
 # A hash (by filename) of a hash (by version) of lines added.
 %file_version_delta = ();
 
 # A hash (by filename) of a hash (by version) of the file state.
 %file_version_state = ();
 
+# A hash (by filename) of a hash (by version) of the author.
+%file_version_author = ();
+
 # A hash (by filename) of the magic branch number.
 %file_branch_number = ();
 
@@ -79,6 +103,9 @@ $filedata = "";
 # A hash (by date) of the total number of files.
 %total_files = ();
 
+# A hash (by date) of a hash (by user) of their total number of lines.
+%total_user_lines = ();
+
 # Flag to indicate if gnuplot is to be used.
 $use_gnuplot = 0;
 
@@ -95,6 +122,9 @@ $gnuplot_linedata = "";
 # The gnuplot output filename to write the output of the file data.
 $gnuplot_filedata = "";
 
+# The gnuplot output filename to write the output of the user data.
+$gnuplot_userdata = "";
+
 # The gnuplot "set term" expression to use when generating output.
 # By default, generate colour png files.
 $gnuplot_setterm = "png color";
@@ -112,6 +142,7 @@ $use_linestyle = 0;
 $osname = $Config{'osname'};
 $windows = (defined $osname && $osname eq "MSWin32") ? 1 : 0;
 
+check_missing_modules();
 process_command_line_arguments();
 get_cvs_statistics();
 analyse_statistics();
@@ -133,10 +164,68 @@ sub quote
     }
     else
     {
-        shell_quote($arg);
+        String::ShellQuote::shell_quote($arg);
     }
 }
 
+###############################################################################
+# Method for writing out help if modules are missing.
+sub check_missing_modules
+{
+    my @missing = ();
+
+    # Load the Date::Manip module.
+    eval
+    {
+	require Date::Manip;
+    };
+    if ($@)
+    {
+	push @missing, 'Date::Manip';
+    }
+
+    # Load the String::ShellQuote module for UNIX platforms.
+    eval
+    {
+	if (! $windows)
+	{
+	    require String::ShellQuote;
+	}
+    };
+    if ($@)
+    {
+	push @missing, 'String::ShellQuote';
+    }
+    
+    # Check if there are any missing modules.
+    return if $#missing == -1;
+
+    # First, output the generic "missing module" message.
+    print "\n";
+    print "Cvsplot requires some Perl modules which are missing " .
+	  "from your system.\n";
+
+    if ($windows) {
+	print "These can be installed by issuing the following commands:\n\n";
+	foreach my $module (@missing) {
+	    $module =~ s/:://g;
+	    print 'C:\> ' . "ppm install $module\n";
+	}
+	print "\n";
+    }
+    else
+    {
+	print "They can be installed by running (as root) the following:\n";
+	foreach my $module (@missing) {
+	    print "   perl -MCPAN -e 'install \"$module\"'\n";
+	}
+	print "\n";
+	print "Modules can also be downloaded from http://www.cpan.org.\n\n";
+    }
+    exit;
+}
+
+
 ###############################################################################
 # Check whether the supplied file is to be examined or not depending on what
 # the user set for the -include and -exclude options.  Return true if the
@@ -217,8 +306,9 @@ sub get_cvs_statistics
 
     # Build up the command string appropriately, depending on what options
     # have been set.
-    my $command = ($rlog_module eq '') ? 'cvs log' :
-	sprintf('cvs -d %s rlog %s', quote($cvsdir), quote($rlog_module));
+    my $command = ($rlog_module eq '') ? "cvs $cvs_global_args log" :
+	sprintf("cvs $cvs_global_args -d %s rlog %s",
+		quote($cvsdir), quote($rlog_module));
 
     print "Executing \"$command\"\n" if $debug;
 
@@ -346,19 +436,23 @@ sub get_cvs_statistics
 		}
 		    
 		$_ = <CVSLOG>;		# Read the "date" line.
-		if (/^date: (\d\d\d\d\/\d\d\/\d\d \d\d:\d\d:\d\d); .* state: (.*);.*lines: \+(\d+) \-(\d+).*$/)
+		if (/^date: (\d\d\d\d\/\d\d\/\d\d \d\d:\d\d:\d\d); .* author: (.*); .* state: (.*);.*lines: \+(\d+) \-(\d+).*$/)
 		{
 		    # Note for some CVS clients, state dead is presented in
 		    # this this way, as the following pattern.
 		    $date = $1;
-		    $state = $2;
-		    $lines_added = $3;
-		    $lines_removed = $4;
-		    $number_lines = $lines_added - $lines_removed;
+		    $author = $2;
+		    $users{$author} = 1;
+		    $state = $3;
+		    $lines_added = $4;
+		    $lines_removed = $5;
+		    $number_lines = $count_lines_changed ?
+			$lines_added + $lines_removed : $lines_added - $lines_removed;
 
 		    $file_version_delta{$working_file}{$revision} =
 			$number_lines;
 		    $file_version_state{$working_file}{$revision} = $state;
+		    $file_version_author{$working_file}{$revision} = $author;
 
 		    if ($file_on_branch)
 		    {
@@ -366,27 +460,34 @@ sub get_cvs_statistics
 			$line_stats{$date}{$working_file} += $number_lines;
 			$state_stats{$date}{$working_file} = $state;
 			$revision_stats{$date}{$working_file} = $revision;
+			$author_stats{$date}{$working_file} = $author;
 		    }
 	        }
-		elsif (/^date: (\d\d\d\d\/\d\d\/\d\d \d\d:\d\d:\d\d); .* state: dead;.*$/)
+		elsif (/^date: (\d\d\d\d\/\d\d\/\d\d \d\d:\d\d:\d\d); .* author: (.*); .* state: dead;.*$/)
 		{
 		    # File has been removed.
 		    $date = $1;
+		    $author = $2;
+		    $users{$author} = 1;
 
 		    $file_version_delta{$working_file}{$revision} = 0;
 		    $file_version_state{$working_file}{$revision} = "dead";
+		    $file_version_author{$working_file}{$revision} = $author;
 		    
 		    if ($file_on_branch)
 		    {
 			$line_stats{$date}{$working_file} = 0;
 			$state_stats{$date}{$working_file} = "dead";
 			$revision_stats{$date}{$working_file} = $revision;
+			$author_stats{$date}{$working_file} = $author;
 		    }
 		}
-		elsif (/^date: (\d\d\d\d\/\d\d\/\d\d \d\d:\d\d:\d\d); .* state: ([^;]*);.*$/)
+		elsif (/^date: (\d\d\d\d\/\d\d\/\d\d \d\d:\d\d:\d\d); .* author: (.*); .* state: ([^;]*);.*$/)
 		{
 		    $date = $1;
-		    my $state = $2;
+		    $author = $2;
+		    $users{$author} = 1;
+		    $state = $3;
 
 		    # Unfortunately, cvs log doesn't indicate the number of
 		    # lines an initial revision is created with, so find this
@@ -408,14 +509,14 @@ sub get_cvs_statistics
 			    print STDERR "Please correct your -cvsdir argument and try again\n";
 			    exit 1;
 			}
-			$lccmd = sprintf('cvs -d %s co -r %s -p %s',
+			$lccmd = sprintf("cvs $cvs_global_args -d %s co -r %s -p %s",
 					 quote($cvsdir),
 					 quote($revision),
 					 quote($2));
 		    }
 		    else
 		    {
-			$lccmd = sprintf('cvs update -r %s -p %s',
+			$lccmd = sprintf("cvs $cvs_global_args update -r %s -p %s",
 					 quote($revision),
 					 quote($relative_working_file));
 		    }
@@ -445,12 +546,14 @@ sub get_cvs_statistics
 		    $file_version_delta{$working_file}{$revision} =
 			$number_lines;
 		    $file_version_state{$working_file}{$revision} = $state;
+		    $file_version_author{$working_file}{$revision} = $author;
 		    
 		    if ($file_on_branch)
 		    {
 			$line_stats{$date}{$working_file} += $number_lines;
 			$state_stats{$date}{$working_file} = "Exp";
 			$revision_stats{$date}{$working_file} = $revision;
+			$author_stats{$date}{$working_file} = $author;
 		    }
 		}
 		else
@@ -462,6 +565,7 @@ sub get_cvs_statistics
 		    print "File \"$working_file\" rev $revision ";
 		    print "delta $file_version_delta{$working_file}{$revision} ";
 		    print "state $file_version_state{$working_file}{$revision}\n";
+		    print "author $file_version_author{$working_file}{$revision}\n";
 		}
 	    }
 	}
@@ -588,6 +692,20 @@ sub analyse_statistics
     # statistics.  Their initial revisions will be the revision that
     # they branched from if we are doing statistics on a branch.
     my %file_revision = ();
+
+    # Keep a record of the current state of a file when gathering
+    # statistics.
+    my %file_state = ();
+
+    # Keep a record of per-user line counts;
+    my %user_total = ();
+
+    # A hash of filenames to a hash of usernames to linecount totals.  This
+    # is maintained as the statistics are processed, so that when files are
+    # removed or re-added, all of the individual contributions are properly
+    # maintained to the correct authors.
+    my %file_author_linecount = ();
+
     if ($branch_tag ne "")
     {
 	foreach $file ( keys %file_branch_number )
@@ -605,21 +723,57 @@ sub analyse_statistics
     # Go through the records in order of earliest to latest.
     foreach $date ( sort keys %line_stats )
     {
-	# Investigate what CVS operations occurred on this date.
+	# Investigate what CVS operations occured on this date.  Record
+	# individual author contributions.
 	foreach $file ( keys %{ $line_stats{$date} } )
 	{
-	    # Update the current revision a file has.
+	    # Update the current revision the file has.
 	    $file_revision{$file} = $revision_stats{$date}{$file};
+	    $author = $author_stats{$date}{$file};
 
 	    # Record if a file was removed or not.
 	    if ($state_stats{$date}{$file} eq "dead")
 	    {
 		delete $files_present{$file};
+
+		# For each author contribution made to this file, deduct them
+		# now.
+		foreach my $innerauthor (keys %{ $file_author_linecount{$file} })
+		{
+		    my $amount = $file_author_linecount{$file}{$innerauthor};
+		    $user_total{$innerauthor} -= $amount;
+		    $total_user_lines{$innerauthor}{$date} = $user_total{$innerauthor};
+		}
 	    }
 	    else
 	    {
 		$files_present{$file} = 1;
+
+		# If the file was removed in the previous commit, need to add
+		# the entire line count broken amongst the appropriate authors,
+		# otherwise just add the increment.
+		my $state = $file_state{$file};
+		if (defined $state && $state eq "dead")
+		{
+		    # For each author contribution made to this file, add them
+		    # to the appropriate authors now.
+		    foreach my $innerauthor (keys %{ $file_author_linecount{$file} })
+		    {
+			my $amount = $file_author_linecount{$file}{$innerauthor};
+			$user_total{$innerauthor} += $amount;
+			$total_user_lines{$innerauthor}{$date} = $amount;
+		    }
+		}
+
+		# Now add in the increment done in this commit.
+		$user_total{$author} += $line_stats{$date}{$file};
+		$file_author_linecount{$file}{$author} +=
+		    $line_stats{$date}{$file};
+		$total_user_lines{$author}{$date} = $user_total{$author};
 	    }
+
+	    # Record the current state of this file.
+	    $file_state{$file} = $state_stats{$date}{$file};
 	}
 
 	# Calculate the total number of files present on this date.
@@ -642,15 +796,23 @@ sub analyse_statistics
     # for now...
     foreach $date ( sort keys %total_lines )
     {
-	$current_date = &ParseDate($date);
-	if (($start_date ne "" && &Date_Cmp($current_date, $start_date) < 0) ||
-	    ($end_date ne "" && &Date_Cmp($current_date, $end_date) > 0))
+	$current_date = Date::Manip::ParseDate($date);
+	if (($start_date ne "" &&
+	     Date::Manip::Date_Cmp($current_date, $start_date) < 0) ||
+	    ($end_date ne "" &&
+	     Date::Manip::Date_Cmp($current_date, $end_date) > 0))
 	{
 	    # This date is before the start date specified by the user, or
 	    # this date is after the end date specified by the user.
 	    # Delete it.
 	    delete $total_lines{$date};
 	    delete $total_files{$date};
+
+	    # Also handle per-user date filtering.
+	    foreach $author ( keys %total_user_lines )
+	    {
+		delete $total_user_lines{$author}{$date};
+	    }
 	}
     }
 }
@@ -662,14 +824,102 @@ sub analyse_statistics
 #
 sub generate_data_files
 {
-    # Write out the line data.
+    # Create a map from user names to ids, and likewise for groups, so that
+    # each distinct user/group is mapped to an id, which is used later for
+    # Gnuplot.
+    my %user_map = ();
+    my %group_map = ();
+    my $index = 0;
+    foreach my $user ( sort keys %total_user_lines )
+    {
+	$user_map{$user} = $index++;
+
+	# If groups are enabled, and the default group is specified, check if
+	# this user needs to be added to it.
+	if ($default_user_group ne '')
+	{
+	    my $found = 0;
+	    foreach my $group ( sort keys %usergroups )
+	    {
+		if (exists $usergroups{$group}{$user})
+		{
+		    $found = 1;
+		    last;
+		}
+	    }
+	    if ($found == 0)
+	    {
+		# Add to default group.
+		$usergroups{$default_user_group}{$user} = 1;
+	    }
+	}
+    }
+    $index = 0;
+    foreach my $group ( sort keys %usergroups )
+    {
+	$group_map{$group} = $index++;
+    }
+
+    # Keep a running record of total linecounts for each group.
+    my %total_group_lines = ();
+
+    # Determine if authors are being filtered.
+    my $filtering_users = scalar keys %filterusers != 0;
+    my $filtering_groups = scalar keys %usergroups != 0;
+
+    # Write out the data for each commit in date order.
     foreach $date ( sort keys %total_lines )
     {
 	print LINEDATA "$date $total_lines{$date}\n";
 	print FILEDATA "$date $total_files{$date}\n";
+
+	# Check if there is any author data to output.
+	if ($userdata ne "")
+	{
+	    foreach my $author ( sort keys %user_map )
+	    {
+		# Only output if it is an author of interest.
+		next if ($filtering_users && $default_user_group eq '' &&
+			 ! exists $filterusers{$author});
+
+		# Update author/group data if defined.
+		my $data = $total_user_lines{$author}{$date};
+		if (defined $data)
+		{
+		    if ($filtering_groups)
+		    {
+			# Find out the group membership for this author.
+			foreach my $group ( sort keys %usergroups )
+			{
+			    if (exists $usergroups{$group}{$author})
+			    {
+				# Author is a member of this group, update the
+				# group stats and update the total count.
+				$total_group_lines{$group}{$author} = $data;
+				my $total = 0;
+				foreach my $user (keys %{ $total_group_lines{$group} })
+				{
+				    $total += $total_group_lines{$group}{$user};
+				}
+				print USERDATA "$date $group " .
+				               "$group_map{$group} " .
+				               "$total\n";
+			    }
+			}
+		    }
+		    else
+		    {
+			# Output author information.
+			print USERDATA "$date $author " .
+			               "$user_map{$author} $data\n";
+		    }
+		}
+	    }
+	}
     }
     close LINEDATA;
     close FILEDATA;
+    close USERDATA if $userdata ne "";
 }
 
 ###############################################################################
@@ -712,6 +962,7 @@ $linestyle_command
 $gnuplot_command
 plot '$linedata' using 1:3 axes x1y1 title 'Lines',\\
      '$filedata' using 1:3 axes x1y2 title 'Files'
+exit
 EOF
 
             # Pipe this command into gnuplot.
@@ -738,12 +989,13 @@ set output '$gnuplot_linedata'
 $linestyle_command
 $gnuplot_command
 plot '$linedata' using 1:3
-set title 'CVS Line Statistics'
+set title 'CVS File Statistics'
 set ylabel 'Number of files'
 set yrange [0:*]
 set output '$gnuplot_filedata'
 $gnuplot_command
 plot '$filedata' using 1:3
+exit
 EOF
 
         # Pipe this command into gnuplot.
@@ -752,6 +1004,64 @@ EOF
         print GNUPLOT $command;
         close GNUPLOT;
     }
+
+    if ($gnuplot_userdata)
+    {
+        # Now build up the plot command, for the specified users.
+	# Determine if authors are being filtered.
+        my $filtering_users = scalar keys %filterusers != 0;
+	my $filtering_groups = scalar keys %usergroups != 0;
+	my $title = 'Per-' . ($filtering_groups ? 'group' : 'user') .
+	    ' CVS Line Statistics';
+
+	# Produce a plot containing a list of per-user stats.
+	$command = <<EOF;
+
+set xdata time
+set timefmt '%Y/%m/%d %H:%M:%S'
+set format x '%m/%y'
+set xlabel 'Date'
+set ylabel 'Number of lines'
+set title '$title'
+set yrange [0:*]
+set key top left
+set terminal $gnuplot_setterm
+set output '$gnuplot_userdata'
+$linestyle_command
+$gnuplot_command
+EOF
+    
+	$command .= "\nplot " ;
+        my $index = 0;
+	if ($filtering_groups)
+	{
+	    foreach my $group ( sort keys %usergroups )
+	    {
+		$command .= ", " if ($index != 0);
+		$command .= "'$userdata' using 1:(\$4 == $index ? \$5 : 1/0) t '$group'";
+		$index++;
+	    }
+	}
+	else
+	{
+	    foreach my $user ( sort keys %total_user_lines )
+	    {
+		if (! $filtering_users || exists $filterusers{$user})
+		{
+		    $command .= ", " if ($index != 0);
+		    $command .= "'$userdata' using 1:(\$4 == $index ? \$5 : 1/0) t '$user'";
+		}
+		$index++;
+	    }
+	}
+	$command .= "\nexit\n";
+
+	# Pipe this command into gnuplot.
+        print "Issuing command:\n\n$command\n\n to gnuplot: $gnuplot\n" if $debug;
+	open(GNUPLOT, "| $gnuplot") || die "Failed to start gnuplot: $!";
+        print GNUPLOT $command;
+        close GNUPLOT;
+    }
 }
 
 ###############################################################################
@@ -766,11 +1076,21 @@ sub process_command_line_arguments
 	    $debug = 1;
 	    $i++;
 	}
+	elsif ($ARGV[$i] eq "-cvs-global-args")
+	{
+	    $cvs_global_args = $ARGV[$i+1];
+	    $i += 2;
+	}
 	elsif ($ARGV[$i] eq "-linestyle")
 	{
 	    $use_linestyle = 1;
 	    $i++;
 	}
+	elsif ($ARGV[$i] eq "-countchangedlines")
+	{
+	    $count_lines_changed = 1;
+	    $i++;
+	}
 	elsif ($ARGV[$i] eq "-include")
 	{
 	    $pattern_include[++$#pattern_include] = 1;
@@ -790,12 +1110,12 @@ sub process_command_line_arguments
 	}
 	elsif ($ARGV[$i] eq "-start")
 	{
-	    $start_date = &ParseDate($ARGV[$i+1]);
+	    $start_date = Date::Manip::ParseDate($ARGV[$i+1]);
 	    $i += 2;
 	}
 	elsif ($ARGV[$i] eq "-end")
 	{
-	    $end_date = &ParseDate($ARGV[$i+1]);
+	    $end_date = Date::Manip::ParseDate($ARGV[$i+1]);
 	    $i += 2;
 	}
 	elsif ($ARGV[$i] eq "-cvsdir")
@@ -818,6 +1138,39 @@ sub process_command_line_arguments
 	    $filedata = $ARGV[$i+1];
 	    $i += 2;
 	}
+	elsif ($ARGV[$i] eq "-userdata")
+	{
+	    $userdata = $ARGV[$i+1];
+	    $i += 2;
+	}
+	elsif ($ARGV[$i] eq "-userlist")
+	{
+	    my @userlist = ();
+	    my $groupname = undef;
+	    if ($ARGV[$i+1] =~ /^(\w+)=(.*)$/)
+	    {
+		# A list of users assigned to a group.
+		$groupname = $1;
+		@userlist = split /,/, $2;
+	    }
+	    else
+	    {
+		# Just a list of individual users to run over.
+		@userlist = split /,/, $ARGV[$i+1];
+	    }
+	    foreach my $username (@userlist)
+	    {
+		$filterusers{$username} = 1;
+		$usergroups{$groupname}{$username} = 1 if defined $groupname;
+	    }
+	    $i += 2;
+	}
+	elsif ($ARGV[$i] eq "-defaultusergroup")
+	{
+	    $default_user_group = $ARGV[$i+1];
+	    $usergroups{$default_user_group} = ();
+	    $i += 2;
+	}
 	elsif ($ARGV[$i] eq "-gnuplot")
 	{
 	    $use_gnuplot = 1;
@@ -836,6 +1189,12 @@ sub process_command_line_arguments
 	    $gnuplot_filedata = $ARGV[$i+1];
 	    $i += 2;
 	}
+	elsif ($ARGV[$i] eq "-gnuplotuserdata")
+	{
+	    $use_gnuplot = 1;
+	    $gnuplot_userdata = $ARGV[$i+1];
+	    $i += 2;
+	}
 	elsif ($ARGV[$i] eq "-gnuplotlinefiledata")
 	{
 	    $use_gnuplot = 1;
@@ -905,6 +1264,20 @@ sub process_command_line_arguments
 	}
     }
 
+    # If -userlist has been specified, make sure -userdata has.
+    if (scalar keys %filterusers != 0 && $userdata eq "")
+    {
+	print "error: -userdata needs to be specified if -userdata is.\n\n";
+	usage();
+    }
+
+    # If -gnuplotuseruserdata has been specified, make sure -userdata has.
+    if ($gnuplot_userdata ne "" && $userdata eq "")
+    {
+	print "error: -userdata needs to be specified if -gnuplotuserdata is.\n\n";
+	usage();
+    }
+
     # If both the start and end dates are specified, check that the start date
     # occurs before the end date.
     if ($start_date ne "" && $end_date ne "" &&
@@ -922,7 +1295,7 @@ sub process_command_line_arguments
 	my $WTR = gensym();
 	my $RDR = gensym();
 	my $ERR = gensym();
-	my $pid = open3($WTR, $RDR, $ERR, "cvs rlog");
+	my $pid = open3($WTR, $RDR, $ERR, "cvs $cvs_global_args rlog");
 	my $deprecated_found = 0;
         while (<$ERR>)
         {
@@ -945,6 +1318,12 @@ sub process_command_line_arguments
 
     open (FILEDATA, ">$filedata")
 	|| die "Failed to create file \"$filedata\": $!";
+    
+    if ($userdata ne "")
+    {
+	open (USERDATA, ">$userdata")
+	    || die "Failed to create file \"$userdata\": $!";
+    }
 }
 	    
 ###############################################################################
@@ -952,17 +1331,22 @@ sub process_command_line_arguments
 #
 sub usage
 {
-    print "cvsplot version 1.6.5 - ";
+    print "cvsplot version 1.7.4 - ";
     print "Copyright David Sitsky: sits\@users.sourceforge.net\n\n";
     print "cvsplot collects statistics from CVS controlled files.\n\n";
     print "usage:\n";
     print "cvsplot.pl -cvsdir <dir> [-rlog <module>] -linedata <file> -filedata <file>\n";
+    print "           [-userdata <file>] [-userlist [groupname=]<user1,user2,...>]*\n";
+    print "           [-defaultusergroup <groupname>]\n";
+    print "           [-cvs-global-args <global-args>]\n";
+    print "           [-countchangedlines]\n";
     print "           [-branch <branch name>] [-start <date>] [-end <date>]\n";
-    print "           {-include <regexp> | -exclude <regexp>} [-linesstyle]\n";
+    print "           {-include <regexp> | -exclude <regexp>} [-linestyle]\n";
     print "           [-gnuplotlinedata <output file for line data>\n";
     print "            -gnuplotfiledata <output file for file data>\n";
     print "            [-gnuplot <path to gnuplot binary>]\n";
     print "            [-gnuplotlinefiledata <output file combined line and file data>]\n";
+    print "            [-gnuplotuserdata <output file for user data>\n";
     print "            [-gnuplotsetterm <gnuplot set terminal expression>]\n";
     print "            [-gnuplotcommand <general gnuplot command>]]\n\n";
     print "See http://cvsplot.sourceforge.net for updates.\n";
